import os
import tarfile
import tempfile
import backstage
from backstage.shortcuts import Act, Venue
from backstage.venue.venue_utils import use_venue


def new_act(venue, actname):
    """create a new Act within a backstage Venue"""
    if isinstance(venue, str):
        try:
            venue = use_venue(venue)
        except:
            venue = None
    if not isinstance(venue, Venue):
        print '%s is not a valid backstage venue' % venue
        return None
    acthome = os.path.join(venue.acts_root, actname)
    if os.path.exists(acthome):
        print 'A folder named %s already exists under %s' % (actname, venue.acts_root)
        return None

    try:
        os.mkdir(acthome)
        copy_act_skel(venue, venue.acts_root, actname)
        keyfile = 'backstage-%s-%s.id' % (venue.venue_name, actname )
        with open(os.path.join(acthome, '.LIVE', keyfile), 'w') as kf:
            kf.write('#%s' % keyfile)
        act = Act(venue, actname)
        s = 'created Backstage Act %s at %s\n' % (actname, acthome)
        s += 'using Act %s' % actname
        print s
        venue.get_acts()
        return act
    except:
        s = 'something went wrong creating act.\n'
        s += 'you need to clean up the garbage because i have yet to do that for you.\n'
        print s
        raise None


def copy_act_skel(venue, actsdir, actname):
    """copy the skeleton files into the Act instance folder"""
    backstage_home = os.path.dirname(os.path.abspath(backstage.__file__))
    act_home = os.path.join(actsdir, actname)
    act_skel = os.path.join(backstage_home, 'skel/act')
    #recursive copy via tar
    tmpfile = tempfile.NamedTemporaryFile()
    tmpfile.close()
    tar_file = tarfile.open(tmpfile.name, 'w')
    cwd = os.getcwd()
    os.chdir(act_skel)
    tar_file.add('.')
    os.chdir(cwd)
    tar_file.close()
    tar_file = tarfile.open(tmpfile.name, 'r')
    tar_file.extractall(act_home)
    tar_file.close()
    os.remove(tmpfile.name)



__author__ = 'walker'
import unittest
import psycopg2

class TestDatabaseFunctionality(unittest.TestCase):

    def setUp(self):
        pass


    def test_connect(self):
        """ Test that we can connect to the database server
        @return:
        """
        conn = psycopg2.connect("dbname=backstage host=127.0.0.1 user=backstage")


    def test_create(self):
        """
        Test that we can create a database within the db server
        @return:
        """


if __name__ == '__main__':
    unittest.main()


"""local wsgi.py file for this Backstage Act instance.
Contents determined explicitly from this file's location.
"""
import os
import sys

#path name for this Act

acts_path, act_name = os.path.split(os.path.dirname(os.path.abspath(__file__)))

sys.path.insert(0,acts_path)
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "%s.settings" % (act_name))
from django.core.wsgi import get_wsgi_application
application=get_wsgi_application()



table = 'uwsgi_vassals'
config = 'dummy text'
uid = 'backstage'
gid = 'adm'

uwsgi_defaults = {
    'master': 'True',
    'processes': '8',
    'http-socket': '127.0.0.1:0',
    'plugins': 'python',
    'harikiri': '20',
    'log-maxsize': '100000',
    'max-requests': '1000',
    'vacuum': 'true',
    'disable-logging': 'true',
    'uid': 'backstage',
    'gid': 'adm',
    'logfile-choawn': 'true',
    'logfile-chmod': '666',
    }

venue_seeds = {
    'home': '{VENUE_HOME}/venv',
    'virtualenv': '{VENUE_HOME}/venv',
    'pythonpath': '{VENUE_ROOT}',
    'chdir': '{VENUE_ROOT}',
    'module': '{VENUE_NAME}.wsgi:application',
    'pidfile': '{VENUE_HOME}/.LIVE/uwsgi-backstage-{VENUE_NAME}.pid',
    'socket': '{VENUE_HOME}/.LIVE/uwsgi-backstage-{VENUE_NAME}.sock',
    'daemonize': '/var/log/uwsgi/uwsgi-backstage-{VENUE_NAME}.log',
}

act_seeds = {
    'home': '{VENUE_HOME}/venv',
    'virtualenv': '{VENUE_HOME}/venv',
    'chdir': '{VENUE_HOME}/acts',
    'pythonpath': '{VENUE_HOME}/acts',
    'module': '{ACT_NAME}.wsgi:application',
    'pidfile': '{VENUE_HOME}/.LIVE/uwsgi-backstage-{VENUE_NAME}-{ACT_NAME}.pid',
    'socket': '{VENUE_HOME}/.LIVE/uwsgi-backstage-{VENUE_NAME}-{ACT_NAME}.sock',
    'daemonize': '/var/log/uwsgi/uwsgi-backstage-{VENUE_NAME}-{ACT_NAME}.log',
}


def format_act_options(act):
    opts = {}
    for k, v in act_seeds.iteritems():
        opts[k] = v.format(VENUE_HOME=act.venue.venue_home, VENUE_NAME=act.venue.name, ACT_NAME=act.name)
    return opts


def build_uwsgi(obj, objtype):
    """
    build a string formatted like a uwsgi.ini file
    @return:
    """
    uwsgi_config = {}
    for k, v in uwsgi_defaults.iteritems():
        uwsgi_config[k] = v


    options = {
        'act': format_act_options,
        'venue': format_venue_options,
        }

    local_options = options[objtype](obj)
    for k, v in local_options.iteritems():
        uwsgi_config[k] = v

    uwsgi_ini = '[uwsgi]\n'
    for k, v in uwsgi_config.iteritems():
        uwsgi_ini += '%s = %s\n' % (k, v)
    return uwsgi_config, uwsgi_ini


def format_venue_options(venue):
    opts = {}
    for k, v in venue_seeds.iteritems():
        opts[k] = v.format(VENUE_HOME=venue.venue_home, VENUE_NAME=venue.venue_name, VENUE_ROOT=venue.venue_root)
    return opts




"""
backstage shortcuts.py
Shortcut imports for convenience.
"""
from backstage.act.act import Act
from backstage.venue.venue import Venue




from backstage.venue.venue_utils import  new_venue, venue_from_cwd
from backstage.act.act_utils import  new_act









#!/usr/bin/python
import os,sys

venue_DB = 'enterprise'

SITE_ID=1
ADMINS = ()
MANAGERS = ADMINS
USE_TZ = True
TIME_ZONE = 'America/Los_Angeles'
LANGUAGE_CODE = 'en-us'
USE_I18N = True
SECRET_KEY = '0p3ffc)q#f-4rv23&amp;yqp-f_p04p(lp(i!&amp;wljvvnk690=!0wo$'

EMAIL_HOST = ''
EMAIL_PORT = 587
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = True

INTERNAL_IPS=('127.0.0.1',)

#location of templates
venue_TEMPLATE_DIRS = ['venue/templates',]


#context processors
venue_TEMPLATE_CONTEXT_PROCESSORS = [
    'backstage.utils.context_processors.settings_constants',
]

#automatically convert spaces to underscores
UNDERSCORIFY=True



import os
import sys

from backstage.shortcuts import Venue
import backstage


def copy_venue_skel(venue_home):
    """Populate a (usually) new backstage venue with the contents of backstage/skel"""
    skeldir = os.path.join(os.path.dirname(backstage.__file__), 'skel/venue')
    if not os.path.exists(skeldir):
        s = 'Source skeleton files for backstage venue not found.  This is a system error'
        print s
        raise
    try:
        #Copy the contents of the 'skel' folder into the venue root
        os.system("cp -rp %s/. %s" % (skeldir, venue_home))

    except:
        s = "ERROR copying the venue skel into %s" % venue_home
        print s
        raise
    return True

def create_venue_uwsgi_file(venue_base, venue_root, venue_name):
    """create the uwsgi ini file for a (usually) new Venue.
    Reads backstage/config/uwsgi.venue.ini.src
    """
    srcfile = os.path.join(os.path.dirname(backstage.__file__), 'conf/uwsgi.venue.ini.src')
    with open(srcfile, 'r') as f:
        srcdata = f.read()
    outfile = os.path.join(venue_root, 'backstage-%s-uwsgi.ini' % venue_name)
    o = open(outfile, 'w')
    o.write(srcdata.format(VENUE_BASE=venue_base, VENUE_ROOT=venue_root, VENUE_NAME=venue_name))
    o.close()
    return


def new_venue(venue_name, venue_root):
        """create a new backstage venue with the given name and located at the specified path"""
        venue_home = os.path.abspath(os.path.join(venue_root, venue_name))
        try:
            os.makedirs(venue_home)
        except:
            print 'Error creating venue.'
            raise

        try:
            with open(os.path.join(venue_home, '__init__.py'), 'w'):
                pass
        except:
            err = 'ERROR with init file'
            print err
            raise
        try:
            copy_venue_skel(venue_home)
        except:
            s = 'Error in copy_venue_skel'
            print s
            raise

        try:
            create_venue_uwsgi_file(venue_root, venue_home, venue_name)
        except:
            s = 'Error in create_venue_wsgi_file'
            print s
            raise

        try:
            venue = use_venue(venue_home)
        except:
            raise

        venue.build_virtualenv()
        s = 'Successfully created Backstage venue %s at %s' % (venue.venue_name, venue.venue_root)
        print s
        return venue


def test_venue_exists(venue_home):
    """Test for the existence of a Backstage venue instance.  Return True or False"""
    if not os.path.exists(venue_home):
        s = 'venue folder at %s does not exist. Terminating' % venue_home
        print s
        return False
    #A keyfile under .LIVE should exist.  Proves this is a backstage venue.  Right now it is empty
    keyfilename = 'backstage-venue.txt'
    keyfile = os.path.join(venue_home, '.LIVE', keyfilename)
    if not os.path.exists(keyfile):
        s = 'Backstage key file file not found'
        print s
        return False
    return True


def use_venue(venue_home):
    """Use an existing Backstage venue.  Returns the venue instance."""
    exists = test_venue_exists(venue_home)
    if not exists:
        s = 'Venue does not exist'
        raise s

    try:
        venue = Venue(venue_home)
        print venue
        dir(venue)
    except:
        print
        raise
    paths = [venue.venue_home, venue.venue_root, ]
    for pth in paths:
        if not pth in sys.path:
            sys.path.append(pth)
    try:
        venue.get_settings()
    except:
        s = 'Could not import settings'
        print s
        raise
    # paramfiles are those with parameterized variables needing to be replaced using the params dict.
    venue.paramfiles = ['site_settings.py', 'theme_settings.py', 'wsgi.py', 'conf/gunicorn_launcher',
                   'conf/nginx.conf', 'templates/index.html',
                   'conf/supervisor.conf', ]
    # this would be better in a db table
    venue.themes = ['default', 'default24', 'fluid', 'container', 'hero', ]
    try:
        venue.connect()
    except:
        s = 'Could not connect to database'
        raise
    s = 'Using Backstage venue %s at %s' % (venue.venue_name, venue.venue_root)
    print s
    return venue


def venue_from_cwd():
    cwd = os.getcwd()
    try:
        v = Venue(cwd)
    except:
        v = None
    return v




import os


#from django.db import models
from django.db.models.signals import pre_init, post_init
from act.act import Act
from venue.venue import Venue


def prevalidate_venue(sender, **kwargs):
    """
    :param sender:
    :param kwargs:
    :return:
    Before we initialize a Venue we validate that it exists on the file system and in the db

    """
    print 'not so fast buddy.'


def postpopulate_venue(sender, **kwargs):
    """populate the venue upon its instantiation
    whether new or per-existent
    """
    venue = kwargs['instance']
    print venue.venue_name


def prevalidate_act(sender, **kwargs):
    """
    Pre-validate an act instance
    @param sender: vessel for passing the venue instance
    @param kwargs: 'instance' contains the venue instance
    @return:
    """
    act = kwargs['instance']
    act.acthome = os.path.join(act.venue.VENUE_ROOT, 'acts', act.actname)
    act.uwsgi_ip = None
    act.uwsgi_port = None
    act.uwsgifile = os.path.join(act.acthome, 'uwsgi.ini')
    if not os.path.exists(act.acthome):
        print 'Act %s does not exist in venue %s' % (act.venue.VENUE_NAME, act.actname)
        raise
    return

def postpopulate_act(sender, **kwargs):
    """
    Populate an act instance with some parameters and methods
    @param sender:
    @param kwargs:
    @return:
    """
    act = kwargs['instance']
    act.get_settings()
    act.uwsgi_linker()
    act.uwsgi_log = act.get_uwsgi_log()
    return

#Execute upon initiation of Venue object.
#Populate some content for the venue
pre_init.connect(prevalidate_act, sender=Act)
post_init.connect(postpopulate_act, sender=Act)
pre_init.connect(prevalidate_venue, sender=Venue)
post_init.connect(postpopulate_venue, sender=Venue)


from django.conf import settings
from django.conf.urls import patterns, include, url
from django.contrib import admin
from mezzanine.core.views import direct_to_template

admin.autodiscover()

urlpatterns = patterns("",
    ("^admin/", include(admin.site.urls)),
    # HOMEPAGE AS STATIC TEMPLATE
    url("^$", direct_to_template, {"template": "index.html"}, name="home"),
    # HOMEPAGE AS AN EDITABLE PAGE IN THE PAGE TREE
    #url("^$", "mezzanine.pages.views.page", {"slug": "/"}, name="home"),
)
# NECESSARY for the DEBUG TOOLBAR pre 1.7
if settings.DEBUG:
    import debug_toolbar
    urlpatterns += patterns('',
        url(r'^__debug__/', include(debug_toolbar.urls)),
    )


#Mezzanine if used as a catch-all must be the final include
mez_urlpatterns = patterns("",
                        ("^", include("mezzanine.urls")),
                        )

urlpatterns += mez_urlpatterns


import os
import sys
import time
import requests

from backstage.utils import uwsgi_portsniffer
from backstage.utils.uwsgi.uwsgi_utils import build_uwsgi

# Choose one of the below as the default uwsgi emperor vassal control:
from backstage.utils.uwsgi.linker_file_ini import start, stop, restart
#from backstage.utils.uwsgi.linker_pg_plugin import start, stop, restart


class Act():

    def __init__(self, venue, actname):
        acthome = os.path.join(venue.acts_root, actname)
        kf = 'backstage-%s-%s.id' % (venue.venue_name, actname)
        keyfile = os.path.join(acthome, '.LIVE', kf)
        if not os.path.exists(keyfile):
            #not a valid act
            return
        self.venue = venue
        self.actname = actname
        self.name = self.actname
        self.acthome = acthome
        self.longname = 'backstage-%s-%s' % (self.venue.name, self.name)
        self.keyfile = keyfile
        self.conn = venue.conn
        self.get_settings()
        self.uwsgi_config, self.uwsgi_ini = build_uwsgi(self, 'act')
        inifile = '%s.ini' % self.longname
        self.uwsgi_file = os.path.join(self.acthome, inifile)
        self.uwsgi_vassal = os.path.join(self.settings.UWSGI_VASSALS, inifile)
        #necessary for file-based uwsgi linking
        self.uwsgi_ip = None
        self.uwsgi_port = None
        if not os.path.exists(self.uwsgi_file):
            with open(self.uwsgi_file, 'w') as f:
                f.write(self.uwsgi_ini)

    def start(self):
        start(self)

    def stop(self):
        stop(self)

    def restart(self):
        restart(self)

    def get_settings(self):
        syspath = sys.path
        sys.path.insert(0, os.path.join(self.venue.venue_home, 'acts'))
        settings = None
        exec('from %s import settings' % self.actname)
        sys.path = syspath
        self.settings = settings
        return


    def get_uwsgi_log(self):
        fo = open(self.uwsgifile, 'r')
        d = fo.readlines()
        fo.close()
        logfile = None
        for line in d:
            line = line.strip()
            if line[0:9] == 'daemonize':
                logfile = line.split('=')[1]
                return logfile
        return logfile

    def get_uwsgi_port(self):
        """Get the uwsgi port using lsof.  Requires that lsof and fuser be suid root"""
        start_port =self.uwsgi_port
        timeout = 10
        nap = 1
        starttime = time.time()
        elapsed = 0
        valid = False
        while not valid and elapsed < timeout:
            try:
                fullport = uwsgi_portsniffer.port_from_lsof(self)
                new_ip, new_port = fullport.split(':')
                if new_port <> start_port:
                    self.uwsgi_ip = new_ip
                    self.uwsgi_port = new_port
                    print 'OK %s:%s' % (new_ip,new_port)
                    valid = True
            except:
                pass
            if not valid:
                time.sleep(nap)
                elapsed = time.time() - starttime
        return

    def connect(self):
        """
        connect to the instance's default database
        @return:
        """
        from backstage.db.db_utils import connect_default
        conn = connect_default(self)

    def sniff_uwsgi_port(self):
        """sniff the uwsgi port from the log file. inefficient but does not require
        root access"""
        ip, port = uwsgi_portsniffer.portsniffer(self.uwsgi_log)
        if port is None:
            print "No port.  Try self.uwsgi_linker(linkmode='link')"
            return
        uwsgi_uri = 'http://%s:%s' % (ip, port)
        try:
            h = requests.head(uwsgi_uri)
            if h.status_code == 200:
                self.uwsgi_ip = ip
                self.uwsgi_port = port
                print str(ip), str(port)
            else:
                print 'request for %s resulted in a status code of %s' % (uwsgi_uri, h.status_code)
                print 'the entire header follows:'
                print h
                if h.status_code == 500:
                    s = 'A status code of 500 means that the port is bound OK but that '
                    s+= 'there is probably a coding error somewhere in the Act '
                    s+= 'which is causing it to fail to load. '
                    s+= 'In your browser - and with DEBUG enabled, visit %s and review the error message' % (uwsgi_uri)
                    print s
                self.uwsgi_ip = None
                self.uwsgi_port = None
        except requests.exceptions.ConnectionError:
            s= 'Failure to load the URI at %s' % (uwsgi_uri)
            s+= 'Hint: this is probably a stale port.\nTry reloading the Act by touching its .ini file.\n'
            s+= 'Or, wait a few more seconds and try "(self).get_uwsgi_port() again'
            print s

            self.uwsgi_ip = None
            self.uwsgi_port = None
            print 'None'
        return


import os

"""HARD-CODE the absolute sqlite path and file name
the file is located in the 'db' subfolder of the venue directory
and is named django_backstage.sq3.
"""
f = os.path.abspath(__file__)
settings_dir = os.path.dirname(f)
venue_dir, settings_name = os.path.split(settings_dir)
venue_parent, venue_name = os.path.split(venue_dir)

DBENGINE = 'django.db.backends.postgresql_psycopg2'
DBHOST = '127.0.0.1'
DBPORT = '5432'
DBUSER = 'backstage'
DBPASS = ''
DBNAME = 'backstage_%s' % venue_name

DATABASES = {}
DATABASES['default'] = {
    'NAME': DBNAME,
    'ENGINE': DBENGINE,
    'HOST': DBHOST,
    'PORT': DBPORT,
    'USER': DBUSER,
    'PASSWORD': DBPASS
}

DEFAULT_DB_ALIAS='default'
DEFAULT_DB = DATABASES['default']



"""
A simple TemplateTag to provide
in-template on-the-fly string underscoring
Usage:  {{ some%20string | underscore }} becomes some_string
"""

from django import template
register = template.Library()

@register.filter
def underscore(string):
    if string is not None:
        return string.replace(' ','_')
    else:
        return ''
